/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstring>
#include "nfanegatedcharclass.h"

NFANegatedCharClass::NFANegatedCharClass()
{
	memset(m_chars, 1, sizeof(m_chars));
}

NFANegatedCharClass::NFANegatedCharClass(const string& chars)
{
	memset(m_chars, 1, sizeof(m_chars));
	addChars(chars);
}

NFANegatedCharClass::NFANegatedCharClass(char from, char to)
{
	memset(m_chars, 1, sizeof(m_chars));
	addChars(from, to);
}

void
NFANegatedCharClass::addChars(const string& chars)
{
	string::const_iterator iter = chars.begin();
	for (; iter != chars.end(); iter++) {
		const char ch = *iter;
		m_chars[ch] = 0;
	}
}

void
NFANegatedCharClass::addChars(char from, char to)
{
	for (int i = from; i <= to; i++) {
		m_chars[i] = 0;
	}
}

list<NFAMove>
NFANegatedCharClass::getMovesFrom(int state) const
{
	list<NFAMove> moves;
	if (state != START_STATE)
		return moves;
	
	for (int i = 0; i < 256; i++) {
		if (m_chars[i]) {
			moves.push_back(NFAMove(i, FINISH_STATE));
		}
	}
	return moves;
}

