/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef BFILTER_H_
#define BFILTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <list>
#include "network.h"
#include "serverinfo.h"
#include "clientrequeststream.h"
#include "serverresponsestream.h"
#include "clientinfo.h"
#include "serverinfo.h"
#include "htmlprocessor.h"
#include "httpclientrequest.h"

using namespace std;

class BFilter
{
	friend class ClientRequestStream;
	friend class ServerResponseStream;
	friend class HTMLProcessor;
public:
	BFilter(Network::Socket csock);
	~BFilter();
	void run();
private:
	void dataForClient(const string& data);
	void dataForServer(const string& data);
	void finishSession(); // throws ChildFinishException
	void flushClientData();
	void switchToServer(const string& host, int port);
	void makeServerConnection();
	void closeServerConnection(bool server_has_closed_it);
	HTMLProcessor& htmlProcessor() { return m_html_processor; }
	ClientInfo& clientInfo() { return m_client_info; }
	ServerInfo& serverInfo() { return m_server_info; }
	ClientRequestStream& requestStream() { return m_request_stream; }
	ServerResponseStream& responseStream() { return m_response_stream; }
	list<HttpClientRequest *>& requestsToBeServed() { return m_requests_to_be_served; }
	list<HttpClientRequest *>& requestsSentToServer() { return m_requests_sent_to_server; }
	bool isRequestSentToServer(const HttpClientRequest *request) const;
	Network::Socket m_csock, m_ssock;
	HTMLProcessor m_html_processor;
	ServerInfo m_server_info;
	ClientInfo m_client_info;
	string m_server_host;
	int m_server_port;
	ClientRequestStream m_request_stream;
	ServerResponseStream m_response_stream;
	string m_data_for_server;
	string m_data_for_client;
	list<HttpClientRequest *> m_requests_to_be_served; // last one may be incomplete
	list<HttpClientRequest *> m_requests_sent_to_server; /* these are not-yet-answered (or not-yet-fully-answered)
	requests that has been sent (the last one may be only partially sent) to the server */
};

#endif

