/*
    BFilter - a web proxy which removes banner ads
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CONFPARSER_H_
#define CONFPARSER_H_

#include <string>
#include <list>
#include <cstdio>
#include "conflistener.h"

using namespace std;

class ConfParser
{
public:
	static void attachListener(ConfListener *listener);
	static void detachListener(ConfListener *listener);
	static void reset();
	static void setFileName(const string& fname) { m_fname = fname; }
	static const string& getFileName() { return m_fname; }
	static bool parse(FILE *input);
	
	static void processSection(const string& section);
	static void processParameter(int key, int value);
	static void processError(const string& errmsg);
private:
	typedef list<ConfListener *> listeners_type;
	static listeners_type m_listeners;
	static string m_fname; // for error messages
};

extern "C" {
	extern int yyparse();
	extern void yyerror(const char *err);
	extern int yylval;
}

#endif

