/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "crc32.h"

unsigned long CRC32::m_crc32_tab[256];
bool CRC32::m_crc32_tab_ready = false;

unsigned long
CRC32::compute(const string& data)
{
	return update(data, 0L);
}

unsigned long
CRC32::update(const string& data, unsigned long crc32)
{
	prepareCRC32Table();
	unsigned long c = crc32 ^ 0xffffffffL;
	string::const_iterator iter = data.begin();
	for (; iter != data.end(); ++iter) {
		c = m_crc32_tab[(c ^ *iter) & 0xff] ^ (c >> 8);
	}
	return c ^ 0xffffffffL;
}

void
CRC32::prepareCRC32Table()
{
	if (m_crc32_tab_ready)
		return;
	unsigned long c;
	int n, k;
	for (n = 0; n < 256; n++) {
		c = (unsigned long)n;
		for (k = 0; k < 8; k++) {
			if (c & 1) {
				c = 0xedb88320L ^ (c >> 1);
			} else {
				c = c >> 1;
			}
		}
		m_crc32_tab[n] = c;
	}
	m_crc32_tab_ready = true;
}

