/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <unistd.h>
#include <iostream>
#include <sstream>
#include <string>
#include <assert.h>
#include "debug.h"

using namespace std;

IPCPortal Debug::m_portal;

void
Debug::setPortal(const IPCPortal& portal)
{
	m_portal = portal;
}

void
Debug::logDebug(const string& data)
{
	IPCMessage msg("debug");
	msg.addParameter(getpid());
	msg.addParameter(data);
	m_portal.sendMessage(msg);
	while (m_portal.hasDataToSend()) {
		m_portal.writeChunk();
	}
}

void
Debug::registerConnection()
{
	IPCMessage msg("registerConnection");
	msg.addParameter(getpid());
	m_portal.sendMessage(msg);
	while (m_portal.hasDataToSend()) {
		m_portal.writeChunk();
	}
	
}

void
Debug::unregisterConnection()
{
	IPCMessage msg("unregisterConnection");
	msg.addParameter(getpid());
	m_portal.sendMessage(msg);
	while (m_portal.hasDataToSend()) {
		m_portal.writeChunk();
	}
}

void
Debug::logTraffic(const string& data, traffic_type ttype)
{
	string method;
	switch (ttype) {
	case TRAF_TO_SERVER:
		method = "trafficToServer";
		break;
	case TRAF_FROM_SERVER:
		method = "trafficFromServer";
		break;
	case TRAF_TO_CLIENT:
		method = "trafficToClient";
		break;
	case TRAF_FROM_CLIENT:
		method = "trafficFromClient";
		break;
	default:
		assert(false);
	}
	IPCMessage msg(method);
	msg.addParameter(getpid());
	msg.addParameter(data);
	m_portal.sendMessage(msg);
	while (m_portal.hasDataToSend()) {
		m_portal.writeChunk();
	}
}

