/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DEBUG_H_
#define DEBUG_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sstream>
#include <string>
#include "ipcportal.h"

using namespace std;

#ifdef DEBUG
#define DEBUGLOG(what) { ostringstream strm; strm << what; Debug::logDebug(strm.str()); }
#define DEBUGLOG2(what) DEBUGLOG(what << " [" << __FILE__ << ":" << __LINE__ << "]")
#define LOG_TRAFFIC_TO_SERVER(what) Debug::logTraffic(what, Debug::TRAF_TO_SERVER)
#define LOG_TRAFFIC_FROM_SERVER(what) Debug::logTraffic(what, Debug::TRAF_FROM_SERVER)
#define LOG_TRAFFIC_TO_CLIENT(what) Debug::logTraffic(what, Debug::TRAF_TO_CLIENT)
#define LOG_TRAFFIC_FROM_CLIENT(what) Debug::logTraffic(what, Debug::TRAF_FROM_CLIENT)
#else
#define DEBUGLOG(what)
#define DEBUGLOG2(what)
#define LOG_TRAFFIC_TO_SERVER(what)
#define LOG_TRAFFIC_FROM_SERVER(what)
#define LOG_TRAFFIC_TO_CLIENT(what)
#define LOG_TRAFFIC_FROM_CLIENT(what)
#endif

class Debug
{
#ifdef DEBUG
public:
#else
private:
#endif
	enum traffic_type { TRAF_TO_SERVER, TRAF_FROM_SERVER, TRAF_TO_CLIENT, TRAF_FROM_CLIENT };
	static void setPortal(const IPCPortal& portal);
	static void registerConnection();
	static void unregisterConnection();
	static void logDebug(const string& data);
	static void logTraffic(const string& data, traffic_type ttype);
protected:
	Debug() {}
private:
	static IPCPortal m_portal;
};

#endif

