/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GIFIMAGEINSPECTOR_H_
#define GIFIMAGEINSPECTOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "abstractimageinspector.h"

using namespace std;

class GIFImageInspector : public AbstractImageInspector
{
public:
	GIFImageInspector();
	virtual void consumeData(const string& data);
	virtual void consumeEOF();
	virtual void reset();
	virtual const string& getAccumulatedData() const { return m_data; }
	virtual bool isInspectionInProgress() const { return m_inspection_in_progress; }
	virtual bool isInspectionFailed() const { return m_inspection_failed; }
	virtual int getWidth() const { return m_width; }
	virtual int getHeight() const { return m_height; }
	virtual bool isAnimated() const { return m_is_animated; }
	static const unsigned char SIGNATURE[3];
private:
	enum state {
		ST_SIGNATURE,
		ST_HEADER,
		ST_GLOBAL_COLOR_TABLE,
		ST_BLOCK_BEGIN,
		ST_IMAGE_DESCRIPTOR,
		ST_LOCAL_COLOR_TABLE,
		ST_IMAGE_DATA,
		ST_IMAGE_DATA_SUBBLOCK_OR_TERMINATOR,
		ST_IMAGE_DATA_SUBBLOCK,
		ST_EXTENSION_BLOCK_BEGIN,
		ST_GRAPHIC_CONTROL_EXTENSION_SUBBLOCK_OR_TERMINATOR,
		ST_GRAPHIC_CONTROL_EXTENSION_SUBBLOCK,
		ST_OTHER_EXTENSION_SUBBLOCK_OR_TERMINATOR,
		ST_OTHER_EXTENSION_SUBBLOCK
	};
	enum delay_state {
		DST_DELAY_NOT_ENCOUNTERED,
		DST_WAITING_FOR_GRAPHIC_BLOCK,
		DST_WAITING_FOR_ANOTHER_GRAPHIC_BLOCK
	};
	state m_state;
	delay_state m_delay_state;
	string m_data;
	bool m_inspection_in_progress;
	bool m_inspection_failed;
	int m_width;
	int m_height;
	bool m_is_animated;
	string::size_type m_cur_pos;
	string::size_type m_cur_block_length;
};

#endif

