/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTMLELEMENTNODE_H_
#define HTMLELEMENTNODE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <map>
#include <set>
#include "htmlnode.h"

using namespace std;

class HTMLElementNode : public HTMLNode
{
public:
	HTMLElementNode(const string& name="dummyname") : m_name(name) {}
	virtual node_type_enum getType() const { return NODE_ELEMENT; }
	virtual const string& getName() const { return m_name; }
	virtual void setName(const string& name) { m_name = name; }
	virtual string toString(bool xhtml, const set<const HTMLNode *> *noclose=0) const;
	virtual string contentsToString(bool xhtml, const set<const HTMLNode *> *noclose=0) const;
	virtual string openingToString(bool xhtml, bool explicit_empty) const;
	virtual void setAttribute(const string& attr, const string& val);
	virtual void setRawAttribute(const string& attr, const string& val);
	virtual void setBooleanAttribute(const string& attr) { m_attributes.insert(attr); m_attr_values.erase(attr); }
	virtual void removeAttribute(const string& attr) { m_attributes.erase(attr); m_attr_values.erase(attr); }
	virtual bool hasAttribute(const string& attr) const
	{ return (m_attributes.find(attr) != m_attributes.end()); }
	virtual string getAttribute(const string& attr) const;
	virtual string getRawAttribute(const string& attr) const;
	virtual bool isBooleanAttribute(const string& attr) const;
	virtual HTMLElementNode* findAncestor(const string& name, bool include_self=false);
	virtual const HTMLElementNode* findAncestor(const string& name, bool include_self=false) const;
protected:
	virtual HTMLNode* clone() const { return new HTMLElementNode(*this); }
private:
	string m_name;
	set<string> m_attributes;
	map<string, string> m_attr_values;
};

#endif

