/*
    BFilter - a web proxy which removes banner ads
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTMLLEXER_H_
#define HTMLLEXER_H_

#include <cctype>
#include <string>
#include <algorithm>
#include "lexer.h"

using namespace std;

class HTMLLexer : public Lexer
{
public:
	enum {
		INITIAL = 0,
		INSIDE_DOCTYPE,
		INSIDE_DOCTYPE_INTERNAL_SUBSET,
		TAG_NAME,
		CLOSING_TAG_NAME,
		INSIDE_TAG,
		INSIDE_CLOSING_TAG,
		ATTR_EQUAL,
		ATTR_VALUE,
		ATTR_VALUE_DQ,
		ATTR_VALUE_SQ,
		ATTR_VALUE_NQ,
		INSIDE_COMMENT,
		CDATA,
		CDATA_COMM,
		SCRIPT_COMM_BROKEN,
		STYLE_COMM_BROKEN
	};
	enum {
		TOK_TAG_BEGIN = 1,
		TOK_TAG_END,
		TOK_CLOSING_TAG_BEGIN,
		TOK_CLOSING_TAG_END,
		TOK_TAG_NAME,
		TOK_CLOSING_TAG_NAME,
		TOK_TAG_EMPTY,
		TOK_ATTR_NAME,
		TOK_ATTR_VALUE,
		TOK_TEXT,
		TOK_COMMENT,
		TOK_CDATA,
		TOK_DOCTYPE
	};
	const string& getCurTag() { return m_cur_tag; }
protected:
	virtual int executeAction(short rule);
private:
	void setCurTag(const string& str) { m_cur_tag = str; }
	void curTagAppend(const string& str) { m_cur_tag.append(str); }
	const string& getCurTagName() const { return m_cur_tag_name; }
	void setCurTagName(const string& name)
	{
		m_cur_tag_name = name;
		transform(m_cur_tag_name.begin(), m_cur_tag_name.end(), m_cur_tag_name.begin(), static_cast<int(*)(int)>(tolower));
	}
	bool isCDATAStarting()
	{
		return (m_cur_tag_name == "script" || m_cur_tag_name == "style");
	}
	bool isCDATAEnding(const string& closing_tag_name)
	{
		string lcname = closing_tag_name;
		transform(lcname.begin(), lcname.end(), lcname.begin(), static_cast<int(*)(int)>(tolower));
		return (lcname == m_cur_tag_name);
	}
	bool isInsideComment()
	{
		return (Lexer::m_scond == INSIDE_COMMENT);
	}
	string m_cur_tag;
	string m_cur_tag_name;
};

#endif
