/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httpclientrequest.h"
#include "utils.h"
#include "debug.h"

using namespace std;

HttpClientRequest::HttpClientRequest(const HttpRequestMetadata& metadata)
:	m_listener(0), m_metadata(metadata), m_orig_request_line(metadata.requestLine()),
	m_absolute_uri(metadata.requestLine().getURI()), m_is_complete(false),
	m_is_already_served(false), m_intercept_handler(IH_NONE)
{
	string path = metadata.requestLine().getURI().getRawPath();
	if (path == "bf.intercept-request/image_substitute") {
		m_intercept_handler = IH_IMAGE_SUBSTITUTE;
		m_intercept_params = metadata.requestLine().getURI().getDecodedQueryComponents();
	} else if (path == "bf.intercept-request/error_substitute") {
		m_intercept_handler = IH_ERROR_SUBSTITUTE;
	} else if (path == "bf.intercept-request/image_analyze") {
		m_intercept_handler = IH_IMAGE_ANALYZE;
		m_intercept_params = metadata.requestLine().getURI().getDecodedQueryComponents();
		m_metadata.requestLine().setURI(URI(getInterceptParam("url")));
	}
	m_is_closing = (m_orig_request_line.getHttpVersion() < HttpVersion::HTTP_1_1
		|| m_metadata.headers().getHeader("Connection").elements().hasElement("close")
		|| m_metadata.headers().getHeader("Proxy-Connection").elements().hasElement("close"));
}

void
HttpClientRequest::appendBodyData(const string& data)
{
	m_body_data += data;
	if (m_listener) {
		m_listener->moreBodyData(data);
	}
}

void
HttpClientRequest::setComplete()
{
	if (m_is_complete) {
		return;
	}
	m_is_complete = true;
	if (m_listener) {
		m_listener->complete();
	}
}

bool
HttpClientRequest::isToBeSent() const
{
	return (getInterceptHandler() != IH_IMAGE_SUBSTITUTE && getInterceptHandler() != IH_ERROR_SUBSTITUTE);
}

bool
HttpClientRequest::isIdempotent() const
{
	const string& method = m_metadata.requestLine().getMethod();
	return (method == "GET" || method == "HEAD" || method == "PUT"
	        || method == "DELETE" || method == "OPTIONS" || method == "TRACE");
}

string
HttpClientRequest::getInterceptParam(const string& name) const
{
	list<NameValuePair>::const_iterator iter = m_intercept_params.begin();
	for (; iter != m_intercept_params.end(); ++iter) {
		if (iter->getName() == name) {
			return iter->getValue();
		}
	}
	return "";
}

