/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPFETCHER_H_
#define HTTPFETCHER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "httpstream.h"
#include "httprequest.h"
#include "httpresponse.h"
#include "httpstatusline.h"
#include "httpheaderscollection.h"
#include "uri.h"

using namespace std;

class HttpFetcher : protected HttpStream
{
public:
	class Exception
	{
	public:
		Exception(const string& message) : m_message(message) {}
		const string& getMessage() const { return m_message; }
	private:
		string m_message;
	};
	HttpFetcher();
	HttpResponse request(const HttpRequest& request); // throws HttpFetcher::Exception
	int getTimeout() const { return m_timeout; }
	void setTimeout(int timeout) { m_timeout = timeout; }
protected:
	virtual bool isRequestStream() const { return false; }
	virtual bool isResponseOnHeadRequest() const { return m_is_head_request; }
	virtual void eventError(int ecode, const char *info=0);
	virtual void eventGotMetadata(const HttpResponseMetadata& metadata);
	virtual void eventGotContentPart(const string& data);
	virtual void eventGotFooters(const HttpHeadersCollection& footers) {}
	virtual void eventMessageEnds(bool connection_needs_closing);
private:
	class ResponseReceivedException {};
	virtual void eventGotMetadata(const HttpRequestMetadata& metadata) {}
	int m_timeout;
	bool m_is_head_request;
	HttpStatusLine m_status_line;
	HttpHeadersCollection m_headers;
	string m_body;
};

#endif

