/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPHEADERELEMENT_H_
#define HTTPHEADERELEMENT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <list>
#include "namevaluepair.h"

using namespace std;

class HttpHeaderElement : public NameValuePair
{
public:
	HttpHeaderElement(const string& name, const string& value="")
	: NameValuePair(name, value) {}
	HttpHeaderElement(const string& name, const string& value, const list<NameValuePair>& params)
	: NameValuePair(name, value), m_params(params) {}
	list<NameValuePair>& parameters() { return m_params; }
	const list<NameValuePair>& parameters() const { return m_params; }
	bool hasParameter(const string& pname) const;
	NameValuePair getParameterByName(const string& pname) const;
	list<NameValuePair> getParametersByName(const string& pname) const;
	string toString() const;
	static string valueToString(const string& value);
	static string quoteValue(const string& value);
private:
	static bool valueRequiresQuoting(const string& value);
	static const char *prepareCharSafetyTable();
	static const char *prepareCharEscapeTable();
	static const char *m_char_safety_table;
	static const char *m_char_escape_table;
	list<NameValuePair> m_params;
};

#endif

