/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstatechunkbody.h"

#include <string>
#include <algorithm>
#include "httpstream.h"

using namespace std;

HttpStateChunkBody::HttpStateChunkBody(HttpStream& owner)
:	m_owner(owner), m_size(0), m_read(0)
{}

void
HttpStateChunkBody::consumeDataChunk(string& pool)
{
	if (m_read < m_size) {
		string::size_type dataread = std::min(pool.length(), m_size-m_read);
		m_owner.eventGotContentPart(pool.substr(0, dataread));
		m_read += dataread;
		pool.erase(0, dataread);
		return;
	}
	
	if (!m_after_body_data.empty()) {
		m_after_body_data += pool;
		string::size_type pos = m_after_body_data.find("\r\n");
		if (pos == string::npos) {
			pool.assign("");
		} else {
			pool.assign(m_after_body_data, pos+2, string::npos);
			m_owner.switchState(HttpStream::STATE_CHUNK_HEADER);
		}
	} else {
		string::size_type pos = pool.find("\r\n");
		if (pos == string::npos) {
			m_after_body_data = pool;
			pool.assign("");
		} else {
			pool.erase(0, pos+2);
			m_owner.switchState(HttpStream::STATE_CHUNK_HEADER);
		}
	}
}

void
HttpStateChunkBody::consumeEOF()
{
	m_owner.eventError(HttpStream::ERR_UNEXPECTED_EOF);
	m_owner.jumpToState(HttpStream::STATE_ERROR);
}

void
HttpStateChunkBody::cleanup()
{
	m_size = 0;
	m_read = 0;
	m_after_body_data.erase();
}

