/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstatechunkheader.h"

#include <string>
#include <sstream>
#include "httpstream.h"

using namespace std;

HttpStateChunkHeader::HttpStateChunkHeader(HttpStream& owner)
:	m_owner(owner)
{}

void
HttpStateChunkHeader::consumeDataChunk(string& pool)
{
	const string* pdata = &pool;
	string accumulated_data;
	string::size_type queued_data_len = m_queued_data.length();
	if (queued_data_len) {
		accumulated_data = m_queued_data+pool;
		pdata = &accumulated_data;
		m_queued_data.erase();
	}
	string::size_type pos = pdata->find("\r\n");
	if (pos != string::npos) {
		string orig_header = pdata->substr(0, pos);
		istringstream strm(orig_header);
		string::size_type size = string::npos;
		strm >> hex >> size;
		if (size == string::npos) {
			m_owner.eventError(HttpStream::ERR_PARSING_CHUNK_HEADER);
			m_owner.jumpToState(HttpStream::STATE_ERROR);
		}
		if (pos+2 > queued_data_len) {
			pool.erase(0, pos+2-queued_data_len);
		}
		if (size == 0) {
			m_owner.switchState(HttpStream::STATE_FOOTERS);
		} else {
			m_owner.m_state_chunk_body.setSize(size);
			m_owner.switchState(HttpStream::STATE_CHUNK_BODY);
		}
		return;
	}
	m_queued_data = *pdata;
	pool.assign("");
}

void
HttpStateChunkHeader::consumeEOF()
{
	m_owner.eventError(HttpStream::ERR_UNEXPECTED_EOF);
	m_owner.jumpToState(HttpStream::STATE_ERROR);
}

void
HttpStateChunkHeader::cleanup()
{
	m_queued_data.erase();
}

