/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "httpstream.h"

HttpStream::HttpStream()
:	m_state_initial(*this), m_state_start_line(*this), m_state_headers(*this),
	m_state_flat_body(*this), m_state_chunk_header(*this), m_state_chunk_body(*this),
	m_state_footers(*this), m_state_simple_response(*this), m_state_error(*this),
	m_state(&m_state_initial), m_inside_consume_fn(false), m_connection_needs_closing(false)
{
	m_state_table[STATE_INITIAL] = &m_state_initial;
	m_state_table[STATE_START_LINE] = &m_state_start_line;
	m_state_table[STATE_HEADERS] = &m_state_headers;
	m_state_table[STATE_FLAT_BODY] = &m_state_flat_body;
	m_state_table[STATE_CHUNK_HEADER] = &m_state_chunk_header;
	m_state_table[STATE_CHUNK_BODY] = &m_state_chunk_body;
	m_state_table[STATE_FOOTERS] = &m_state_footers;
	m_state_table[STATE_SIMPLE_RESPONSE] = &m_state_simple_response;
	m_state_table[STATE_ERROR] = &m_state_error;
}

void
HttpStream::consumeDataChunk(const string& data)
{
	string pool = data;
	try {
		while (!pool.empty()) {
			m_inside_consume_fn = true;
			m_state->consumeDataChunk(pool);
			m_inside_consume_fn = false;
		}
	} catch (ChangeStateException& e) {
		m_inside_consume_fn = false;
		switchState(e.getState());
	} catch (...) {
		m_inside_consume_fn = false;
		switchState(STATE_INITIAL);
		throw;
	}
}

void
HttpStream::consumeEOF()
{
	try {
		m_inside_consume_fn = true;
		m_state->consumeEOF();
		m_inside_consume_fn = false;
	} catch (ChangeStateException& e) {
		m_inside_consume_fn = false;
		switchState(e.getState());
	} catch (...) {
		m_inside_consume_fn = false;
		switchState(STATE_INITIAL);
		throw;
	}
}

void
HttpStream::resetState()
{
	jumpToState(STATE_INITIAL);
}

bool
HttpStream::isInitialState() const
{
	// gcc 2.x needs this cast
	return (m_state == (HttpState *)&m_state_initial);
}

bool
HttpStream::isErrorState() const
{
	// gcc 2.x needs this cast
	return (m_state == (HttpState *)&m_state_error);
}

void
HttpStream::switchState(State state)
{
	m_state->cleanup();
	m_state = m_state_table[state];
}

void
HttpStream::jumpToState(State state)
{
	if (m_inside_consume_fn) {
		throw ChangeStateException(state);
	} else {
		switchState(state);
	}
}

