/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef HTTPVERSION_H_
#define HTTPVERSION_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "utils.h"

using namespace std;

class HttpVersion
{
public:
	HttpVersion(int major, int minor) : m_major(major), m_minor(minor) {}
	int getMajor() const { return m_major; }
	int getMinor() const { return m_minor; }
	string toString() const;
	bool operator==(const HttpVersion& other) const;
	bool operator!=(const HttpVersion& other) const;
	bool operator<(const HttpVersion& other) const;
	bool operator>(const HttpVersion& other) const;
	bool operator>=(const HttpVersion& other) const;
	bool operator<=(const HttpVersion& other) const;
	static HttpVersion HTTP_0_9;
	static HttpVersion HTTP_1_0;
	static HttpVersion HTTP_1_1;
private:
	int m_major;
	int m_minor;
};

inline string
HttpVersion::toString() const
{
	return string("HTTP/")+num2str(m_major)+"."+num2str(m_minor);
}

inline bool
HttpVersion::operator==(const HttpVersion& other) const
{
	return (m_major == other.m_major && m_minor == other.m_minor);
}

inline bool
HttpVersion::operator!=(const HttpVersion& other) const
{
	return !(*this == other);
}

inline bool
HttpVersion::operator<(const HttpVersion& other) const
{
	if (m_major < other.m_major) {
		return true;
	} else if (m_major > other.m_major) {
		return false;
	} else {
		return (m_minor < other.m_minor);
	}
}

inline bool
HttpVersion::operator>(const HttpVersion& other) const
{
	if (m_major > other.m_major) {
		return true;
	} else if (m_major < other.m_major) {
		return false;
	} else {
		return (m_minor > other.m_minor);
	}
}

inline bool
HttpVersion::operator>=(const HttpVersion& other) const
{
	return (*this == other || *this > other);
}

inline bool
HttpVersion::operator<=(const HttpVersion& other) const
{
	return (*this == other || *this < other);
}

#endif

