/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "imageinspector.h"

#include <cstring>
#include "jpegimageinspector.h"
#include "gifimageinspector.h"
#include "pngimageinspector.h"

ImageInspector::ImageInspector()
:	m_state(ST_SIGNATURE), m_type(T_UNKNOWN), m_delegate(0)
{}

ImageInspector::~ImageInspector()
{
	if (m_delegate) {
		delete m_delegate;
	}
}

void
ImageInspector::consumeData(const string& data)
{
	if (m_state == ST_DELEGATION) {
		m_delegate->consumeData(data);
		return;
	}
	m_data += data;
	if (m_state == ST_UNKNOWN_TYPE) {
		return;
	}
	
	// m_state == STATE_SIGNATURE
	bool not_jpeg = false;
	bool not_gif = false;
	bool not_png = false;
	if (m_data.length() >= sizeof(JPEGImageInspector::SIGNATURE)) {
		if (!memcmp(m_data.data(), JPEGImageInspector::SIGNATURE, sizeof(JPEGImageInspector::SIGNATURE))) {
			m_delegate = new JPEGImageInspector();
			m_delegate->consumeData(m_data);
			m_data.erase();
			m_type = T_JPEG;
			m_state = ST_DELEGATION;
			return;
		} else {
			not_jpeg = true;
		}
	}
	if (m_data.length() >= sizeof(GIFImageInspector::SIGNATURE)) {
		if (!memcmp(m_data.data(), GIFImageInspector::SIGNATURE, sizeof(GIFImageInspector::SIGNATURE))) {
			m_delegate = new GIFImageInspector();
			m_delegate->consumeData(m_data);
			m_data.erase();
			m_type = T_GIF;
			m_state = ST_DELEGATION;
			return;
		} else {
			not_gif = true;
		}
	}
	if (m_data.length() >= sizeof(PNGImageInspector::SIGNATURE)) {
		if (!memcmp(m_data.data(), PNGImageInspector::SIGNATURE, sizeof(PNGImageInspector::SIGNATURE))) {
			m_delegate = new PNGImageInspector();
			m_delegate->consumeData(m_data);
			m_data.erase();
			m_type = T_PNG;
			m_state = ST_DELEGATION;
			return;
		} else {
			not_png = true;
		}
	}
	if (not_jpeg && not_gif && not_png) {
		m_state = ST_UNKNOWN_TYPE;
	}
}

void
ImageInspector::consumeEOF()
{
	if (m_state == ST_DELEGATION) {
		m_delegate->consumeEOF();
	} else {
		m_state = ST_UNKNOWN_TYPE;
	}
}

const string&
ImageInspector::getAccumulatedData() const
{
	if (m_state == ST_DELEGATION) {
		return m_delegate->getAccumulatedData();
	} else {
		return m_data;
	}
}

void
ImageInspector::reset()
{
	if (m_delegate) {
		delete m_delegate;
	}
	m_delegate = 0;
	m_state = ST_SIGNATURE;
	m_type = T_UNKNOWN;
	m_data.erase();
}

bool
ImageInspector::isInspectionInProgress() const
{
	if (m_state == ST_DELEGATION) {
		return m_delegate->isInspectionInProgress();
	} else if (m_state == ST_SIGNATURE) {
		return true;
	} else { // m_state == ST_UNKNOWN_TYPE
		return false;
	}
}

bool
ImageInspector::isInspectionFailed() const
{
	if (m_state == ST_DELEGATION) {
		return m_delegate->isInspectionFailed();
	} else {
		return (m_state == ST_UNKNOWN_TYPE);
	}
}

int
ImageInspector::getWidth() const
{
	if (m_state == ST_DELEGATION) {
		return m_delegate->getWidth();
	} else {
		return 0;
	}
}

int
ImageInspector::getHeight() const
{
	if (m_state == ST_DELEGATION) {
		return m_delegate->getHeight();
	} else {
		return 0;
	}
}

bool
ImageInspector::isAnimated() const
{
	if (m_state == ST_DELEGATION) {
		return m_delegate->isAnimated();
	} else {
		return false;
	}
}

