/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IMAGERESPONSEFILTER_H_
#define IMAGERESPONSEFILTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include "abstractresponsefilter.h"
#include "httpresponsemetadata.h"
#include "imageinspector.h"

using namespace std;

class ImageResponseFilter : public AbstractResponseFilter
{
public:
	ImageResponseFilter(int ad_rule, int width, int height, const string& target,
	                    bool external_link, bool external_img, bool suspicious_link,
			    bool suspicious_img, bool redir, bool cookies);
	virtual void consumeMetadata(HttpResponseMetadata& metadata) { m_metadata = &metadata; }
	virtual void consumeDataChunk(const string& data);
	virtual void consumeEOF();
	virtual bool isMetadataReady() const;
	virtual bool hasDataAvailable() const;
	virtual void readDataChunk(string& collector);
	virtual bool isEOF() const { return m_eof; }
private:
	enum state { ST_INSPECTING, ST_FORWARDING, ST_SUBSTITUTING };
	bool isAd() const;
	state m_state;
	bool m_eof;
	HttpResponseMetadata *m_metadata;
	string m_processed_data;
	ImageInspector m_inspector;
	int m_ad_rule;
	int m_width;
	int m_height;
	string m_target;
	bool m_external_link;
	bool m_external_img;
	bool m_suspicious_link;
	bool m_suspicious_img;
	bool m_redir;
	bool m_cookies;
	
};

#endif

