/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "ipcmessage.h"

string
IPCMessage::serialize() const
{
	ostringstream strm;
	strm << m_method.length() << ':' << m_method << ':' << nParams() << ':';
	vector<string>::const_iterator iter = m_params.begin();
	for (; iter != m_params.end(); ++iter) {
		strm << iter->length() << ':' << *iter << ':';
	}
	return strm.str();
}

void
IPCMessage::unserialize(const string& str)
{
	clearParams();
	istringstream strm(str);
	string::size_type mlen = 0;
	strm >> mlen;
	strm.get();
	m_method = readBytes(strm, mlen);
	strm.get();
	int nr_params = 0;
	strm >> nr_params;
	strm.get();
	for (int i = 0; i < nr_params; i++) {
		string::size_type size = 0;
		strm >> size;
		strm.get();
		m_params.push_back(readBytes(strm, size));
		strm.get();
	}
}

string
IPCMessage::readBytes(istream& strm, string::size_type bytes)
{
	string res;
	string::size_type todo = bytes;
	char buf[200];
	while (todo > 0) {
		string::size_type cnt = strm.read(buf, (sizeof(buf) < todo ? sizeof(buf) : todo)).gcount();
		if (cnt == 0)
			break;
		todo -= cnt;
		res.append(buf, buf+cnt);
	}
	return res;
}

