/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sstream>
#include <unistd.h>
#include "ipcportal.h"
#include "syscall.h"

const IPCMessage *
IPCPortal::firstReceived() const
{
	return (m_receive_queue.empty() ? 0 : &m_receive_queue.front());
}

IPCMessage
IPCPortal::readMessage()
{
	if (m_receive_queue.empty()) {
		return IPCMessage();
	} else {
		IPCMessage ret(m_receive_queue.front());
		m_receive_queue.pop();
		return ret;
	}
}

void
IPCPortal::sendMessage(const IPCMessage& msg)
{
	if (m_to_send.empty()) {
		string serialized = msg.serialize();
		ostringstream strm;
		strm << serialized.length() << ':' << serialized;
		m_to_send = strm.str();
	} else {
		m_send_queue.push(msg);
	}
}

void
IPCPortal::readChunk(const string& data)
{
	string::size_type pos = 0;
	string::size_type len = data.length();
	while (pos < len) {
		if (m_receiving_header) {
			string::size_type fpos = data.find(':', pos);
			if (fpos == string::npos) {
				m_rec_header += data;
				return;
			} else {
				m_rec_header.append(data, pos, fpos-pos);
				istringstream strm(m_rec_header);
				m_rec_data_size = 0;
				strm >> m_rec_data_size;
				m_receiving_header = false;
				m_rec_header.erase();
				pos = fpos+1;
				continue;
			}
		} else {
			if (len-pos < m_rec_data_size-m_rec_data.length()) {
				m_rec_data.append(data, pos, len-pos);
				return;
			} else {
				string::size_type rest = m_rec_data_size-m_rec_data.length();
				m_rec_data.append(data, pos, rest);
				IPCMessage msg;
				msg.unserialize(m_rec_data);
				m_receive_queue.push(msg);
				m_receiving_header = true;
				m_rec_data.erase();
				pos += rest;
				continue;
			}
		}
	}
}

bool
IPCPortal::readChunk()
{
	if (m_rfd == -1 || m_wfd == -1) {
		return true;
	}
	char buf[8192];
	int res = 0;
	SYSCALL(res = read(m_rfd, buf, sizeof(buf)));
	if (res <= 0)
		return false;
	readChunk(string(buf, buf+res));
	return true;
}

bool
IPCPortal::writeChunk()
{
	if (m_rfd == -1 || m_wfd == -1) {
		m_to_send.erase();
		while (!m_send_queue.empty()) {
			m_send_queue.pop();
		}
		return true;
	}
	if (!m_to_send.empty()) {
		int res = 0;
		SYSCALL(res = write(m_wfd, m_to_send.c_str(), m_to_send.length()));
		if (res < 0)
			return false;
		m_to_send.erase(0, res);
	}
	if (m_to_send.empty() && !m_send_queue.empty()) {
		string serialized = m_send_queue.front().serialize();
		m_send_queue.pop();
		ostringstream strm;
		strm << serialized.length() << ':' << serialized;
		m_to_send = strm.str();
	}
	return true;
}

