/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "jpegimageinspector.h"
#include <cstring>

const unsigned char JPEGImageInspector::SIGNATURE[3] = { 0xff, 0xd8, 0xff };

JPEGImageInspector::JPEGImageInspector()
:	m_state(ST_SIGNATURE), m_inspection_in_progress(true),
	m_inspection_failed(false), m_width(0), m_height(0)
{}

void
JPEGImageInspector::consumeData(const string& data)
{
	m_data += data;
	if (!m_inspection_in_progress) {
		return;
	}
	while (true) {
		if (m_state == ST_SIGNATURE) {
			if (m_data.length() >= sizeof(SIGNATURE)) {
				if (memcmp(m_data.data(), SIGNATURE, sizeof(SIGNATURE))) {
					m_inspection_in_progress = false;
					m_inspection_failed = true;
					return;
				} else {
					m_state = ST_MARKER_OR_PADDING;
				}
				m_cur_pos = sizeof(SIGNATURE);
			} else {
				break;
			}
		}
		if (m_state == ST_MARKER_OR_PADDING) {
			if (m_data.length() >= m_cur_pos+1) {
				unsigned char byte = (unsigned char)m_data[m_cur_pos];
				m_cur_pos += 1;
				if (byte == 0xff) {
					continue;
				} else {
					if (byte >= 0xc0 && byte <= 0xcf && byte != 0xc4 && byte != 0xc8 && byte != 0xcc) {
						m_state = ST_SOF;
					} else if (byte == 0xd9 || byte == 0xda) {
						m_inspection_in_progress = false;
						m_inspection_failed = true;
						return;
					} else {
						m_state = ST_SKIP_VAR;
					}
				}
			} else {
				break;
			}
		}
		if (m_state == ST_SKIP_VAR) {
			if (m_data.length() >= m_cur_pos+2) {
				unsigned char byte1 = m_data[m_cur_pos];
				unsigned char byte2 = m_data[m_cur_pos+1];
				m_cur_block_length = ((unsigned int)byte1<<8)+(unsigned int)byte2;
				if (m_cur_block_length < 2) {
					m_inspection_in_progress = false;
					m_inspection_failed = true;
					return;
				}
				m_state = ST_SKIP_VAR_DATA;
			} else {
				break;
			}
		}
		if (m_state == ST_SKIP_VAR_DATA) {
			if (m_data.length() >= m_cur_pos+m_cur_block_length) {
				m_state = ST_MARKER_OR_PADDING;
				m_cur_pos += m_cur_block_length;
			} else {
				break;
			}
		}
		if (m_state == ST_SOF) {
			if (m_data.length() >= m_cur_pos+2) {
				unsigned char byte1 = m_data[m_cur_pos];
				unsigned char byte2 = m_data[m_cur_pos+1];
				m_cur_block_length = ((unsigned int)byte1<<8)+(unsigned int)byte2;
				if (m_cur_block_length < 8) {
					m_inspection_in_progress = false;
					m_inspection_failed = true;
					return;
				}
				m_state = ST_SOF_DATA;
			} else {
				break;
			}
		}
		if (m_state == ST_SOF_DATA) {
			if (m_data.length() >= m_cur_pos+m_cur_block_length) {
				const unsigned char *bytes = (const unsigned char *)m_data.data()+m_cur_pos;
				m_height = ((unsigned int)bytes[3]<<8)+((unsigned int)bytes[4]);
				m_width = ((unsigned int)bytes[5]<<8)+((unsigned int)bytes[6]);
				m_inspection_in_progress = false;
				return;
			} else {
				break;
			}
		}
	}
}

void
JPEGImageInspector::consumeEOF()
{
	if (m_inspection_in_progress) {
		m_inspection_in_progress = false;
		m_inspection_failed = true;
	}
}

void
JPEGImageInspector::reset()
{
	m_state = ST_SIGNATURE;
	m_data.erase();
	m_inspection_in_progress = true;
	m_inspection_failed = false;
	m_width = 0;
	m_height = 0;
	m_cur_pos = 0;
	m_cur_block_length = 0;
}

