/*
    BFilter - a web proxy which removes banner ads
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef NETWORKERROR_H_
#define NETWORKERROR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>

using namespace std;

class NetworkError
{
public:
	typedef enum {
		ERR_NONE = 0,
		ERR_OTHER,
		ERR_UNKNOWN_HOST,
		ERR_CONNECTION_REFUSED,
		ERR_CONNECT_TIMEOUT,
		ERR_NETWORK_UNREACHABLE
	} error_codes;
	NetworkError() : m_err_code(ERR_NONE), m_err_msg() {}
	NetworkError(error_codes err_code, const string& err_msg);
	bool isOK() const { return (m_err_code == ERR_NONE); }
	void resetState() { m_err_code = ERR_NONE; m_err_msg.erase(); }
	void setState(error_codes err_code, const string& err_msg);
	int getErrorCode() const { return m_err_code; }
	const string& getErrorMessage() const { return m_err_msg; }
	void setErrorCode(error_codes err_code) { m_err_code = err_code; }
	void setErrorMessage(const string& err_msg) { m_err_msg = err_msg; }
private:
	error_codes m_err_code;
	string m_err_msg;
};

#endif
