/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "pngimageinspector.h"
#include <cstring>

const unsigned char PNGImageInspector::SIGNATURE[8] = { 0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a };

PNGImageInspector::PNGImageInspector()
:	m_state(ST_SIGNATURE), m_inspection_in_progress(true),
	m_inspection_failed(false), m_width(0), m_height(0)
{}

void
PNGImageInspector::consumeData(const string& data)
{
	m_data += data;
	if (!m_inspection_in_progress) {
		return;
	}
	if (m_state == ST_SIGNATURE) {
		if (m_data.length() >= sizeof(SIGNATURE)) {
			if (memcmp(m_data.data(), SIGNATURE, sizeof(SIGNATURE))) {
				m_inspection_in_progress = false;
				m_inspection_failed = true;
				return;
			} else {
				m_state = ST_HEADER;
			}
		} else {
			return;
		}
	}
	if (m_state == ST_HEADER) {
		if (m_data.length() >= 24) {
			const unsigned char *bytes = (const unsigned char *)m_data.data();
			m_width = ((unsigned int)bytes[16]<<24)+((unsigned int)bytes[17]<<16)+
				((unsigned int)bytes[18]<<8)+(unsigned int)bytes[19];
			m_height = ((unsigned int)bytes[20]<<24)+((unsigned int)bytes[21]<<16)+
				((unsigned int)bytes[22]<<8)+(unsigned int)bytes[23];
			m_inspection_in_progress = false;
		}
	}
}

void
PNGImageInspector::consumeEOF()
{
	if (m_inspection_in_progress) {
		m_inspection_in_progress = false;
		m_inspection_failed = true;
	}
}

void
PNGImageInspector::reset()
{
	m_state = ST_SIGNATURE;
	m_data.erase();
	m_inspection_in_progress = true;
	m_inspection_failed = false;
	m_width = 0;
	m_height = 0;
}

