/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2003  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef URI_H_
#define URI_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <list>
#include "namevaluepair.h"

using namespace std;

class URI
{
public:
	URI(const string& str);
	URI(const URI& base, const URI& relative);
	virtual ~URI() {}
	static string escape(const string& str, const char *allow_extra=0);
	static string unescape(const string& str, bool decode_plus=false);
	bool isGood() const { return m_is_good; }
	bool isAbsolute() const { return m_is_absolute; }
	bool isRelativePath() const { return m_is_relative_path; }
	bool hasAuthority() const { return m_has_authority; }
	bool hasUserInfo() const { return m_has_user_info; }
	bool hasQuery() const { return m_has_query; }
	bool hasFragment() const { return m_has_fragment; }
	const string& getScheme() const { return m_scheme; }
	void setScheme(const string& scheme);
	const string& getRawSchemeSpecificPart() const { return m_raw_scheme_specific_part; }
	void setRawSchemeSpecificPart(const string& str) { parseSchemeSpecificPart(str); }
	const string& getRawAuthority() const { return m_raw_authority; }
	string getDecodedAuthority() const { return unescape(m_raw_authority); }
	void setRawAuthority(const string& authority);
	void removeAuthority();
	const string& getRawUserInfo() const { return m_raw_user_info; }
	string getDecodedUserInfo() const { return unescape(m_raw_user_info); }
	void setRawUserInfo(const string& user_info);
	const string& getHost() const { return m_host; }
	void setHost(const string& host);
	int getPort() const { return m_port; }
	int guessPort() const;
	void setPort(int port);
	const string& getRawPath() const { return m_raw_path; }
	string getDecodedPath() const { return unescape(m_raw_path); }
	void setAbsoluteRawPath(const string& path);
	const string& getRawQuery() const { return m_raw_query; }
	string getDecodedQuery() const { return unescape(m_raw_query, true); }
	void setRawQuery(const string& query);
	void removeQuery();
	list<NameValuePair> getRawQueryComponents(char separator='&') const;
	list<NameValuePair> getDecodedQueryComponents(char separator='&') const;
	const string& getRawFragment() const { return m_raw_fragment; }
	string getDecodedFragment() const { return unescape(m_raw_fragment); }
	void setRawFragment(const string& fragment);
	void removeFragment();
	void makeRelative(); // remove scheme and authority
	string toString() const;
private:
	void parseSchemeSpecificPart(const string& str);
	void updateSchemeSpecificPart();
	void parseAuthority(const string& str);
	void updateAuthority();
	bool validateScheme(const string& str) const;
	bool validateHost(const string& str) const;
	static const string ALNUM;
	bool m_is_good;
	bool m_is_absolute;
	bool m_is_relative_path;
	bool m_has_authority;
	bool m_has_user_info;
	bool m_has_query;
	bool m_has_fragment;
	string m_scheme;
	string m_raw_scheme_specific_part;
	string m_raw_authority;
	string m_raw_user_info;
	string m_host;
	int m_port;
	string m_raw_path;
	string m_raw_query;
	string m_raw_fragment;
};

#endif

